/* This file can be redistributed and used under GNU Library General Public
 * License. (c) 2000 Micha Moskal <malekith@topnet.pl>. 
 * $Id: v.h,v 1.14 2001/02/03 17:00:36 malekith Exp $ */

#ifndef _V_V_H
#define _V_V_H

#define _DONT_INCLUDE_DIRECTLY
#include <v/config.h>
#include <v/utils.h>
#include <v/block.h>

XCPT(XWidget, XRecover);

#define COLOR(fg,bg) (fg|(bg<<4))
enum COLORS { 
	BLACK, BLUE, GREEN, CYAN, RED, MAGENTA, BROWN, LIGHTGRAY, 
	DARKGRAY, LIGHTBLUE, LIGHTGREEN, LIGHTCYAN, LIGHTRED,
	LIGHTMAGENTA, YELLOW, WHITE
};

class Object {
public:
	const char *name;
	Object() { name = "Object"; }
	virtual ~Object() { }
};

class Event {
public:
	enum {
		dead,
		keyboard,
		mouse,
		command,
	} type;
	union {
		int data;
		struct MouseState { 
			enum {
				focus,		// left down
				execute,	// left up
				start_drag,
				drag,
				end_drag,
				popup,		// right up
			} event;
			int x, y; 
		} m;
	};
	void *user_data;
	
	Event() : type(dead), data(0), user_data(0) { }
	void kill() { type = dead; }
	bool is_alive() { return type != dead; }
	bool is_dead() { return type == dead; }
};

// ----------------------------------------------------------------------
// colorset stuff
// ----------------------------------------------------------------------

struct ColorSetInit {
	const char *name;
	int color;
};

class ColorSet : public Object {
	void *hash_p;
public:
	int get_color(const String &name);
	ColorSet(void *p) : hash_p(p) { }
	~ColorSet();
	static ColorSet *make(ColorSetInit *p);
	static void init();
};


// -----------------------------------------------------------------------
// rectangle
// -----------------------------------------------------------------------
class Rect {
public:
	int x, y, w, h;
	Rect(int X = 0, int Y = 0, int W = 0, int H = 0) :
		x(X), y(Y), w(W), h(H) { }
	void move_to(int xx, int yy) { x = xx; y = yy; }
	void move_by(int dx, int dy) { x += dx; y += dy; }
	void resize_to(int ww, int hh) { w = ww; h = hh; }
	void resize_by(int dw, int dh) { w += dw; h += dh; }
	void set(int X, int Y, int W, int H) { x = X; y = Y; w = W; h = H; }
	void grow(int dx, int dy, int dw, int dh)
		{ x += dx; y += dy; w += dw; h += dh; }
	void draw_frame_at(int xx, int yy, int color, bool fill = false);
	void draw_frame(int color, bool fill = false)
		{ draw_frame_at(x, y, color, fill); }
	bool has_inside(int xx, int yy)
		{ return xx >= x && yy >= y && xx - x < w && yy - y < h; }
};

// -----------------------------------------------------------------------
// widgets
// -----------------------------------------------------------------------

// only Form offsprings can have active, children != 0
class Widget : public Object {
	void do_loop();
public:
	Widget *next, *parent, *active, *children;
	Rect pos;
	ColorSet *colorset;
	bool dead, visible;
	String help;
	
	virtual void draw() { }
	virtual bool remove_dead_children() { return false; }
	virtual void time_hook() 
		{ if (active) active->time_hook(); }
	
	bool is_active(Widget *p)
		{ return is_active() && p == active; }
	bool is_active()
		{ return parent == 0 || parent->is_active(this); }
	int pos_x() { return pos.x + (parent ? parent->pos_x() : 0); }
	int pos_y() { return pos.y + (parent ? parent->pos_y() : 0); }
	virtual bool can_get_focus() { return visible; }
	virtual void popup() { if (parent) parent->popup(); }
	virtual bool is_accel(Event &) { return false; }
	Widget *prev();
	virtual void handle_event(Event &e);
	virtual void event_loop();
	
	virtual void activate_hook() { }
	virtual void deactivate_hook() { }
	
	virtual void child_command(int cmd) 
		{ if (parent) parent->child_command(cmd); }
	Widget()
		: next(null), parent(null), active(null), children(null),
		colorset(0), dead(false), visible(true)
		{ 
			name = "Widget";
		}
	ColorSet *get_colorset();
	int color(const String &name) 
		{ return get_colorset()->get_color(name); }
};

class ShortCut {
public:
	int key, cmd;
	ShortCut(int k = 0, int func = 0) : key(k), cmd(func) { }
	bool operator == (const ShortCut &r) const
		{ return r.key == key; }
};

#define v_add_sc(k,c) shortcuts.push(ShortCut(k,c))

class Form : public Widget {
	Widget *widget_at(int x, int y);
	void handle_mouse(Event &e);
protected:
	Widget *drag_owner;
public:
	Block<ShortCut> shortcuts;
	
	virtual void draw();
	virtual void handle_event(Event &e);
	virtual bool remove_dead_children();
	
	virtual void kill_child(Widget *ptr);
	void add_child(Widget *ptr);
	void focus_next();
	void focus_prev();
	
	virtual void activate(Widget *ptr);
	
	Form() : drag_owner(0) { name = "Form"; }
	virtual ~Form();
};


void v_alt_print(const String &s, int cn, int ca, int len = -1);
int v_alt_strlen(const String &s);

int v_message_box(const String &text, const String &title = "Message Box",
		  const String &buttons = "~O~k");

void v_init();
void v_event_loop();
void v_die();
extern ColorSet *v_default_colorset; 

enum {
	ev_none,
	ev_quit = 1,
	ev_repaint,
	ev_menu_start,
	ev_menu_stop,
	ev_user = 0x10000,
};

#include <v/window.h>

inline void add_window(Window *w)
{
	v_root_window.add_child(w);
	v_root_window.activate(w);
}

inline void kill_window(Window *w)
{
	v_root_window.kill_child(w);
	v_root_window.draw();
}

inline void send_event(Event &e)
{
	v_root_window.handle_event(e);
}

inline void send_command(int no)
{
	Event e;
	e.type = Event::command;
	e.data = no;
	v_root_window.handle_event(e);
}

inline int execute_dialog(Dialog *d, bool cleanup = true)
{
	int r;
	add_window(d);
	d->dead = false;
	d->modal_result = 0;
	r = d->execute();
	if (cleanup)
		kill_window(d);
	return r;
}

extern void (*v_frame_proc)();

#include <v/button.h>
#undef _DONT_INCLUDE_DIRECTLY

// configuration:
int v_get_int_conf(const String &r);
String v_get_string_conf(const String &r);
bool _v_alt_checkkey(const String & s, int k);

// to be redefined to respond menu events
void v_handle_command(int data);

#endif /* _V_V_H */

